//File: MergerEngine.cpp

#include "MergerEngine.h"

#include <iostream>
using std::cout;
using std::endl;

BlockRect GetBoundary(BlockRect & r, int dir) {
	BlockRect t = r;
	if( dir == DIR_UP    ) t.min.z = r.max.z;
	if( dir == DIR_FRONT ) t.min.y = r.max.y;
	if( dir == DIR_RIGHT ) t.min.x = r.max.x;
	
	if( dir == DIR_DOWN ) t.max.z = r.min.z;
	if( dir == DIR_BACK ) t.max.y = r.min.y;
	if( dir == DIR_LEFT ) t.max.x = r.min.x;
	return t;
}

BlockRect ShiftRect(BlockRect & r, int dir) {
	BlockPos zero(0,0,0);
	BlockPos add = AdjacentBlock(zero,dir);
	BlockPos hack1 = r.min + add;
	BlockPos hack2 = r.max + add;
	r.min = hack1;
	r.max = hack2;
	return r;
}

void MergerEngine::RemoveRect(BlockRect & r) {
	for(int dir = 1; dir <= 6; dir++) {
		BlockRect b = GetBoundary(r, dir);
		boundary_to_rect.erase(b);
	}
	rects.erase(r);
}

bool MergerEngine::CanMergeRects(BlockRect & a, BlockRect & b) {
	if( a.max.x + 1 == b.min.x ||
		b.max.x + 1 == a.min.x )
	{
		if( a.min.y == b.min.y &&
			a.max.y == b.max.y &&
			a.min.z == b.min.z &&
			a.max.z == b.max.z ) return true;
	}
	if( a.max.y + 1 == b.min.y ||
		b.max.y + 1 == a.min.y )
	{
		if( a.min.x == b.min.x &&
			a.max.x == b.max.x &&
			a.min.z == b.min.z &&
			a.max.z == b.max.z ) return true;
	}
	if( a.max.z + 1 == b.min.z ||
		b.max.z + 1 == a.min.z )
	{
		if( a.min.x == b.min.x &&
			a.max.x == b.max.x &&
			a.min.y == b.min.y &&
			a.max.y == b.max.y ) return true;
	}
	return false;
}

void MergerEngine::AddRect(BlockRect & r, string & data) {
	//Note: This needs to be more clever to take different data into
	//account.
	for(int dir = 1; dir <= 6; dir++) {
		BlockRect b = GetBoundary(r, dir);
		BlockRect sb = ShiftRect(b, dir);
		if( boundary_to_rect.find(sb) != boundary_to_rect.end() ) {
			//There is an adjacent block_rect.
			map<BlockRect, string>::iterator adj_itr = boundary_to_rect[sb];
			BlockRect adj_rect = adj_itr->first;
			string adj_data = adj_itr->second;
			if( adj_data == data ) {
				//The adjacent block_rect has the same data parameters.
				if( CanMergeRects(r, adj_rect) ) {
					BlockRect mr = SumBlockRects(r, adj_rect);
					//cout << "merging: " << mr.min.x << " " << mr.min.y << " " << mr.min.z << " ";
					//cout << mr.max.x << " " << mr.max.y << " " << mr.max.z << endl;
					RemoveRect(adj_rect);
					AddRect(mr, data);
					return;
				}
			}
		}
	}
	//Could not suscessfully merge.
	//Simple adding the unmerged block_rect:
	rects[r] = data;
	map<BlockRect, string>::iterator rect_itr = rects.find( r );
	for(int dir = 1; dir <= 6; dir++) {
		BlockRect b = GetBoundary(r, dir);
		boundary_to_rect[b] = rect_itr;
	}
}

void MergerEngine::AddBlock(BlockPos & pos, string & data) {
	//cout << "block: " << pos.x << " " << pos.y << " " << pos.z << endl;
	BlockRect r(pos, pos);
	AddRect(r, data);
}

int MergerEngine::NumRects() {
	return rects.size();
}

map<BlockRect, string>::iterator MergerEngine::GetRawData() { return rects.begin(); }

map<BlockRect, string>::iterator MergerEngine::GetRawDataEnd() { return rects.end(); }

//more!!!
