//File: StringHelpers.cpp
//Purpose: Provide simple string functions

#include "StringHelpers.h"

#include <string>
#include <sstream>
#include <cmath>
using std::string;

bool IsDigit(char c) {
	if(c >= '0' && c <= '9') return true;
	return false;
}

//Is natural number.
bool IsNaturalNumber(const string & str, int & result) {
	if(str.size() == 0) return false;
	if(str.size() == 1) {
		result = str[0] - '0';
		return true;
	}
	if(str[0] == '0') {
		//Could be something like "0342"
		return false;
	}
	for(unsigned int i = 0; i < str.size(); i++) {
		if( !IsDigit(str[i]) ) return false;
	}
	result = 0;
	for(unsigned int i = 0; i < str.size(); i++) {
		result *= 10;
		result += str[i] - '0';
	}
	return true;
}

bool IsInteger(const string & str, int & result) {
	if( str.size() == 0 ) return false;
	if( str[0] == '-' ) {
		string remaining;
		for(int i = 1; i < str.size(); i++) {
			remaining += str[i];
		}
		int value;
		if( IsNaturalNumber(remaining, value) == false ) return false;
		result = -value;
		return true;
	} else {
		return IsNaturalNumber(str,result);
	}	
}

//Can be a natural number with 0 or more leading 0's.
bool IsNaturalNumberWithLeadingZeros(const string & str, int & result) {
	if(str.size() == 0) return false;
	for(unsigned int i = 0; i < str.size(); i++) {
		if( !IsDigit(str[i]) ) return false;
	}
	result = 0;
	for(unsigned int i = 0; i < str.size(); i++) {
		result *= 10;
		result += str[i] - '0';
	}
	return true;
}

bool IsFloatingPoint(const string & str, float & result) {
	if(str.size() == 0) return false;
	bool found_decimal_point = false;
	bool found_minus_sign = false;
	string left_side, right_side;
	for(unsigned int i = 0; i < str.size(); i++) {
		if( str[i] == '-' ) {
			if ( i != 0 ) return false;
			found_minus_sign = true;
			continue;
		}
		if( IsDigit(str[i]) ) {
			if(found_decimal_point) right_side.push_back( str[i] );
			else left_side.push_back( str[i] );
			continue;
		}
		if( str[i] == '.' ) {
			if( found_decimal_point ) return false;
			found_decimal_point = true;
			continue;
		}
		return false;
	}
	if( found_decimal_point ) {
		if( left_side.empty() && right_side.empty() ) return false;
		if( right_side.empty() ) right_side = "0"; //To be nice.
		if( left_side.empty() ) left_side = "0"; //To be nice.
		int temp_left, temp_right;
		if( IsNaturalNumber(left_side, temp_left) == false ) return false;
		if( IsNaturalNumberWithLeadingZeros(right_side, temp_right) == false ) return false;
		result = (float)temp_left + ( (float)temp_right / pow(10.0f, (float)right_side.size() ) );
		if( found_minus_sign ) result *= (-1.0f);
		return true;
	} else {
		int temp;
		bool is_natural_number = IsNaturalNumber(left_side, temp);
		result = (float)temp;
		if( found_minus_sign ) result *= (-1.0f);
		return is_natural_number;
	}
}

//Not for newlines
inline bool _IsWhiteSpace(char c) {
	if(c == ' ' || c == '\t') return true;
	return false;
}

void ExtractStringsFromLine(const string & line, vector<string> & strs) {
	strs.clear();
	bool adding_to_string = false;
	string current_string;
	for(unsigned int i = 0; i < line.size(); i++) {
		char c = line[i];
		if(c == 13 || c == 10) continue; //Totally ignore newlines, so the strings won't be contaminated.
		if( _IsWhiteSpace(c) ) {
			if( adding_to_string ) {
				adding_to_string = false;
				strs.push_back(current_string);
				current_string.clear();
			}
		} else {
			if( adding_to_string == false ) adding_to_string = true;
			current_string.push_back( c );
		}
	}
	if( adding_to_string ) {
		strs.push_back( current_string );
	}
}

//Gets rid of all 13 and 10 characters from the string.
void RemoveNewLineCharacters(string & str) {
	string temp;
	for(unsigned int i = 0; i < str.size(); i++) {
		char c = str[i];
		if(c != 10 && c != 13) temp += c;
	}
	str = temp;
}

bool StartsWith(string & str, string & substr) {
	if( substr.size() > str.size() ) return false;
	return substr == str.substr(0, substr.size());
}

bool EndsWith(string & str, string & substr) {
	if( substr.size() > str.size() ) return false;
	return substr == str.substr(str.size() - substr.size(), substr.size());
}
