//vector.h
//Purpose: Declare 3d-Vector class
//Note: Do not change the size of the vector class
//(it should have three float members).  This is
//because it is used in a directx vertex struct.

#ifndef Vector_h_
#define Vector_h_

#include <cmath>

class Vector {
public:
	//Default constructor
	Vector() {}

	//Simple constructor
	Vector(float x, float y, float z) {
		this->x = x;
		this->y = y;
		this->z = z;
	}

	bool operator ==(const Vector & b) const {
		return (x == b.x && y == b.y && z == b.z);
	}

	bool operator !=(const Vector & b) const {
		return (x != b.x || y != b.y || z != b.z);
	}

	//Vector addition
	Vector operator +(const Vector & b) const {
		Vector temp(x + b.x, y + b.y, z + b.z);
		return temp;
	}

	//Vector addition
	Vector operator +=(const Vector & b) {
		x += b.x;
		y += b.y;
		z += b.z;
		return *this;
	}

	//Vector subtraction
	Vector operator -(const Vector & b) const {
		Vector temp(x - b.x, y - b.y, z - b.z);
		return temp;
	}

	//Vector subtraction
	Vector operator -=(const Vector & b) {
		x -= b.x;
		y -= b.y;
		z -= b.z;
		return *this;
	}

	//Scalar multiplication
	Vector operator *(float c) const {
		Vector temp(x*c, y*c, z*c);
		return temp;
	}

	//Scalar multiplication
	Vector operator *=(float c) {
		x *= c;
		y *= c;
		z *= c;
		return *this;
	}

	//Scalar division
	Vector operator /(float c) const {
		Vector temp(x/c, y/c, z/c);
		return temp;
	}

	//Scalar division
	Vector operator /=(float c) {
		x /= c;
		y /= c;
		z /= c;
		return *this;
	}

public:
	float x, y, z;
};

//Computes the Dot product of two vectors.
float inline Dot(const Vector & a, const Vector & b) {
	return a.x * b.x + a.y * b.y + a.z * b.z;
}

//Computes the Cross product of two vectors.
Vector inline Cross(const Vector & a, const Vector & b) {
	Vector temp(
		a.y * b.z - a.z * b.y,
		a.z * b.x - a.x * b.z,
		a.x * b.y - a.y * b.x);
	return temp;
}

//Worry about overflowing.
float inline LengthSquared(const Vector & v) {
	return v.x * v.x + v.y * v.y + v.z * v.z;
}

//Worry about overflowing.
float inline Length(const Vector & v) {
	return sqrt( v.x * v.x + v.y * v.y + v.z * v.z );
}

void inline Normalize(Vector & v) {
	float length_squared = v.x * v.x + v.y * v.y + v.z * v.z;
	v /= sqrt(length_squared);
}

Vector inline ProjectionOfBOnA(Vector & a, Vector & b) {
	return a * ( Dot(a, b) / LengthSquared(a) );
}

Vector inline ProjectPointOnPlane(Vector & point, Vector & normal) {
	return point - ProjectionOfBOnA(normal, point);
}

Vector inline BounceVector(Vector & surface_normal, Vector & vel, float elasticity) {
	Vector a = vel * (-1.0f);
	Vector b = ProjectionOfBOnA(surface_normal, a);
	Vector c = b - a;
	return c + b * elasticity;
}

#endif
